package oqbe.actions;

import java.lang.reflect.InvocationTargetException;

import odra.execution.OdraDB;
import oqbe.exporters.ASTExporter;
import oqbe.exporters.OQBEModel;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.progress.IProgressService;
import org.eclipse.uml2.uml.Variable;

import plugin.views.RepositoryBrowser;

import vide.ast.abstracts.AbstractNode;
import vide.converters.ASTToVideCode;
import vide.converters.VideCodeToAST;
import editor.plugin.TextualEditorActivator;

public class ExecuteOQBE implements IEditorActionDelegate {
	private OQBEModel oqbeModel;
	private IEditorPart editor;
	
	@Override
	public void setActiveEditor(IAction action, IEditorPart targetEditor) {
		this.editor = targetEditor;
	}
	
	@Override
	public void run(IAction action) {
		IRunnableWithProgress op = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException {
				try {
					doFinish(monitor);
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				} catch (Exception e) {
					throw new InvocationTargetException(e);					
				} finally {
					monitor.done();
				}
			}
		};
		IProgressService service = PlatformUI.getWorkbench().getProgressService();
		try {
			service.run(true, false, op);
		} catch (InterruptedException e) {
		} catch (InvocationTargetException e) {
			Throwable realException = e.getTargetException();
			MessageDialog.openError(Display.getDefault().getActiveShell(), "Error", realException.getMessage());
			return;
		}
	}
	
	private void doFinish(IProgressMonitor monitor) throws Exception {
		monitor.beginTask("Execute OQBE.", 2);
		monitor.setTaskName("Generating AST...");
		FileEditorInput fileInput = (FileEditorInput) this.editor.getEditorInput();
		try {
			this.oqbeModel = new OQBEModel(fileInput);
		} catch (Exception e) {
			e.printStackTrace();
		}
		ASTExporter generator = new ASTExporter();
		final AbstractNode node = generator.generate(this.oqbeModel);
		monitor.worked(1);
		
		// Extra generation step: convert AST to code and then back to AST
		String code = ASTToVideCode.convert(node, false);
		final AbstractNode node2 = VideCodeToAST.convert(code);
		
		
		monitor.setTaskName("Executing...");
		final OdraDB db = TextualEditorActivator.getDefault().getOdraDB();
		
		// FIXME: this assumes the first context name is self
		final org.eclipse.uml2.uml.Class contextClass = (org.eclipse.uml2.uml.Class) ((Variable) this.oqbeModel.getPackage().getContext().getNames().get(0)).getType();
		//final org.eclipse.uml2.uml.Class contextClass = this.oqbeModel.getPackage().getContext().getOperation().getClass_();
		
		// Solves thread access problem in the Odra Results view
		editor.getSite().getShell().getDisplay().asyncExec(new Runnable() {
		public void run() {
			try {
				// node AST is generated from OQBE -> AST
				// node2 AST is generated from OQBE -> AST -> Text -> AST
				db.executeCode(contextClass, node2);
			} catch (Exception e) {
			}
		}
		});
		monitor.worked(2);
	}
	
	@Override
	public void selectionChanged(IAction action, ISelection selection) {
		// TODO Auto-generated method stub
		
	}
}