package oqbe.actions;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.logging.Level;

import oqbe.exporters.ASTExporter;
import oqbe.exporters.OQBEModel;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.part.FileEditorInput;

import tools.LogWriter;
import vide.ast.abstracts.AbstractNode;
import vide.converters.ASTToVideCode;

public class TestOcl implements IEditorActionDelegate {

	//@Override
	public void setActiveEditor(IAction action, IEditorPart targetEditor) {
	}

	@Override
	public void selectionChanged(IAction action, ISelection selection) {
	}
	
	@Override
	public void run(IAction action) {		
		FileEditorInput fileInput = (FileEditorInput) PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor().getEditorInput();
		try {
			OQBEModel rep  = new OQBEModel(fileInput);
			ASTExporter generator = new ASTExporter();
			AbstractNode node = generator.generate(rep);
			if (node != null) {
				String code = null;
				Level currLogLevel = LogWriter.getLogger().getLevel();
				LogWriter.getLogger().setLevel(Level.OFF);								
				try {
					/*
					AsStringOptions options = new AsStringOptions();
					options.turnOffAll();
					options.showCode = true;
					options.showUmlElement = true;
					System.out.println("AST:");
					System.out.println(node.asString("", options));
					*/
					code = ASTToVideCode.convert(node, false);
				} finally {
					LogWriter.getLogger().setLevel(currLogLevel);
				}
				String fileName = writeFile(fileInput, code);
				openFile(fileName);
			}
		} catch (Exception e) {
			e.printStackTrace();
			MessageDialog.openError(Display.getDefault().getActiveShell(), "Error", e.getMessage());
		}		
	}

    protected String writeFile(FileEditorInput fileInput, String dataString) throws IOException {
    	IPath path = fileInput.getFile().getLocation().removeFileExtension().addFileExtension("vide");
    	String fileName = path.toOSString();
    	File file = new File(fileName);
    	if (file.exists())
    		file.delete();
		PrintWriter writer = new PrintWriter(new BufferedWriter(new FileWriter(file)));
		writer.print(dataString);
		writer.flush();
		writer.close();
		return fileName;
    }
    
    protected void openFile(String fullFileName) {
    	String fileSep = System.getProperty("file.separator");
    	String pathName = fullFileName.substring(0, fullFileName.lastIndexOf(fileSep));
    	String fileName = fullFileName.substring(fullFileName.lastIndexOf(fileSep) + 1);
    	
    	IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
    	IContainer container = root.getContainerForLocation(new Path(pathName));
		final IFile file = container.getFile(new Path(fileName));	
		Display.getDefault().getActiveShell().getDisplay().asyncExec(new Runnable() {
    		public void run() {    			
    			IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
    			try {
    				IDE.openEditor(page, file, true, true);
    			} catch (PartInitException e) {
    				e.printStackTrace();
    			}
    		}
    	});
    }

}