package oqbe.actions.contributions;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IViewActionDelegate;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.progress.IProgressService;
import org.eclipse.uml2.uml.Model;

import plugin.views.RepositoryBrowser;
import OQBE.diagram.part.OQBEmodelDiagramEditor;

public class LoadRepositoryResource implements IViewActionDelegate {
	private RepositoryBrowser repositoryBrowser;
	
	@Override
	public void init(IViewPart view) {
		if (view instanceof RepositoryBrowser) {
			this.repositoryBrowser = (RepositoryBrowser) view;
		}
	}

	@Override
	public void run(IAction action) {
		final IEditorPart editorPart = this.repositoryBrowser.getEditor();
		final Model model = this.repositoryBrowser.getModel();
		if (editorPart instanceof OQBEmodelDiagramEditor && model != null) {
			IRunnableWithProgress op = new IRunnableWithProgress() {
				public void run(IProgressMonitor monitor) throws InvocationTargetException {
					try {
						doFinish(editorPart, model, monitor);
					} catch (CoreException e) {
						throw new InvocationTargetException(e);
					} catch (Exception e) {
						throw new InvocationTargetException(e);					
					} finally {
						monitor.done();
					}
				}
			};
			IProgressService service = PlatformUI.getWorkbench().getProgressService();
			try {
				service.run(true, false, op);
			} catch (InterruptedException e) {
			} catch (InvocationTargetException e) {
			}
		} else if (model == null) {
			MessageDialog.openError(Display.getDefault().getActiveShell(), "Error", "No UML model loaded in the Repository."); 
		} else if(!(editorPart instanceof OQBEmodelDiagramEditor)) {
			MessageDialog.openError(Display.getDefault().getActiveShell(), "Error", "No OQBE diagram opened in the editor.");
		}
	}
	
	private void doFinish(IEditorPart editorPart, Model model, IProgressMonitor monitor) throws Exception {
		if (editorPart instanceof OQBEmodelDiagramEditor) {
			if (((OQBEmodelDiagramEditor) editorPart).getEditingDomain() != null) {
				monitor.beginTask("Load UML model.", 2);
				monitor.setTaskName("Loading resource...");
				try {
					ResourceSet resource = ((OQBEmodelDiagramEditor) editorPart).getEditingDomain().getResourceSet();
					monitor.worked(1);
					resource.getResource(URI.createURI(this.repositoryBrowser.getModelUriString()), true);
					monitor.worked(2);
				} catch (RuntimeException exception) {
				}
			}
		}
	}
	
	@Override
	public void selectionChanged(IAction action, ISelection selection) {
		// TODO Auto-generated method stub
	}
}
