package oqbe.actions.contributions;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.PlatformUI;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.Variable;

import plugin.views.RepositoryBrowser;
import vide.ast.abstracts.AbstractNode;
import vide.converters.VideCodeToAST;
import vide.typechecker.ContextChecker;
import vide.visitors.VisitorException;
import OQBE.Context;
import OQBE.DiagramPackage;
import OQBE.Example;
import OQBE.OQBEFactory;
import OQBE.diagram.part.OQBEmodelCreationWizard;
import OQBE.diagram.part.OQBEmodelDiagramEditorUtil;
import editor.plugin.textual.VideTextualEditor;

public class NewQqbeDiagram implements IEditorActionDelegate {
	private IEditorPart editor;
	private RepositoryBrowser repositoryBrowser;
	
	@Override
	public void setActiveEditor(IAction action, IEditorPart targetEditor) {
		this.editor = targetEditor;
	}
	
	@Override
	public void run(IAction action) {
		if (this.editor instanceof VideTextualEditor) {
			Model model = null;
			this.repositoryBrowser = RepositoryBrowser.getInstance();
			model = this.repositoryBrowser.getModel();
			
			if (model != null) {
				String code = ((VideTextualEditor) this.editor).getText();
				//System.out.println(code);
				AbstractNode abstractNode = null;
				try {
					abstractNode = VideCodeToAST.convert(code);
					abstractNode.updateChildPointers();	// is this necessary?
				} catch (Exception e) {
				}
				
				//System.out.println(abstractNode.asString());
				ContextChecker contextChecker = new ContextChecker(model);
				if (abstractNode != null) {
					try {
						contextChecker.update(abstractNode);
					} catch (VisitorException e) {
					}
				}
				//System.out.println(abstractNode.asString());
				//System.out.println(contextChecker.contextNames.properties);
				//System.out.println(contextChecker.contextNames.parameters);
				
				
				/*
				TextualCodePrintingVisitor visitor = new TextualCodePrintingVisitor();			
				try {			
					abstractNode.accept(visitor, "");
				} catch (VisitorException e) {
					e.printStackTrace();
				}*/
				//System.out.println(visitor.output.toString());
				
				if (contextChecker.contextNames != null) {
					DiagramPackage diagramPackage = OQBEFactory.eINSTANCE.createDiagramPackage();
					this.createContext(diagramPackage, contextChecker);
					diagramPackage.getContext().setFilePath(((IFileEditorInput) this.editor.getEditorInput()).getFile().getFullPath().toString());
					OQBEmodelDiagramEditorUtil.setDiagramPackage(diagramPackage);
				} else if (this.repositoryBrowser.getContext() != null) {
					Element element = this.repositoryBrowser.getContext();
					DiagramPackage diagramPackage = OQBEFactory.eINSTANCE.createDiagramPackage();
					this.createContext(diagramPackage, element);
					diagramPackage.getContext().setFilePath(((IFileEditorInput) this.editor.getEditorInput()).getFile().getFullPath().toString());
					OQBEmodelDiagramEditorUtil.setDiagramPackage(diagramPackage);
				}
				
				IFileEditorInput input = (IFileEditorInput) this.editor.getEditorInput();
				OQBEmodelCreationWizard wizard = new OQBEmodelCreationWizard();
				wizard.init(PlatformUI.getWorkbench(), new StructuredSelection(input));
				WizardDialog dialog = new WizardDialog(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), wizard);
				dialog.open();
				
				//FIXME load resource when there is no context
				/*
				if (contextChecker.contextNames == null) {
					// Is there a better way to get the ResourceSet?
					ResourceSet resource = ((OQBEmodelDiagramEditor) PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor().getEditorSite().getPart()).getEditingDomain().getResourceSet();
					resource.getResource(URI.createURI(this.repositoryBrowser.getModelUriString()), true);
				}*/
			} else {
				MessageDialog.openError(Display.getDefault().getActiveShell(), "Error", "No UML model loaded in the Repository.");
			}
		}
	}
	
	private void createContext(DiagramPackage diagramPackage, ContextChecker contextChecker) {
		Context context = OQBEFactory.eINSTANCE.createContext();
		diagramPackage.setContext(context);
		
		context.setOperation(contextChecker.contextNames.operation);
		
		if (contextChecker.contextNames.selfClass != null) {
			Variable variable = UMLFactory.eINSTANCE.createVariable();
			variable.setName("self");
			variable.setType(contextChecker.contextNames.selfClass);
			context.getNames().add(variable);
			//if (variable.getType().eClass() == UMLPackage.Literals.CLASS) {
				//this.createExample(diagramPackage, variable);
			//}
		}
		
		for (Variable v : contextChecker.contextNames.variables) {
			context.getNames().add(v);
			//if (v.getType().eClass() == UMLPackage.Literals.CLASS) {
				//this.createExample(diagramPackage, v);
			//}
		}
	}
	
	private void createContext(DiagramPackage diagramPackage, Element element) {
		Context context = OQBEFactory.eINSTANCE.createContext();
		diagramPackage.setContext(context);
		if (element instanceof Operation) {
			Operation operation = (Operation) element;
			context.setOperation(operation);
			if (operation.getClass_() != null) {
				Variable variable = UMLFactory.eINSTANCE.createVariable();
				variable.setName("self");
				variable.setType(operation.getClass_());
				context.getNames().add(variable);
			}
		} else if (element instanceof org.eclipse.uml2.uml.Class) {
			org.eclipse.uml2.uml.Class classElement = (org.eclipse.uml2.uml.Class) element;
			Variable variable = UMLFactory.eINSTANCE.createVariable();
			variable.setName("self");
			variable.setType(classElement);
			context.getNames().add(variable);
		}
	}
	
	private void createExample(DiagramPackage diagramPackage, Variable contextName) {
		Example example = OQBEFactory.eINSTANCE.createExample();
		example.setContextName(contextName);
		diagramPackage.getExamples().add(example);
	}
	
	@Override
	public void selectionChanged(IAction action, ISelection selection) {
		// TODO Auto-generated method stub

	}
}