package oqbe.dialogs;

import org.eclipse.core.resources.IProject;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.PlatformUI;

public class GenerateOclDialog extends WizardPage {
	private Text fileText;
	private String fileString = "";
	private Text macroText;
	private String macroString = "";
	
	public GenerateOclDialog() {
		super("Generate OCl");
		setTitle("Generate OCL");
		setDescription("Generates an OCL macro from the OQBE diagram");
	}
	
	public void createControl(Composite parent) {
		Composite topLevel = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout(3, false);
		topLevel.setLayout(layout);
		
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		GridData gridDataText = new GridData(GridData.HORIZONTAL_ALIGN_END);
		gridDataText.widthHint = 280;
		GridData gridDataMacro = new GridData();
		gridDataMacro.widthHint = 120;
		
		Label fileLabel = new Label(topLevel, SWT.NONE);
		fileLabel.setText("File path");
		
		fileText = new Text(topLevel, SWT.SINGLE | SWT.BORDER);
		fileText.setLayoutData(gridDataText);
		fileText.setText(this.fileString);
		
		Button fileButton = new Button(topLevel, SWT.PUSH);
		fileButton.setLayoutData(gridData);
		fileButton.setText("Browse");
		
		Label macroLabel = new Label(topLevel, SWT.NONE);
		macroLabel.setText("Macro name");
		
		macroText = new Text(topLevel, SWT.SINGLE | SWT.BORDER);
		macroText.setLayoutData(gridDataMacro);
		macroText.setText(this.macroString);
		
		fileButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				String[] filterExtensions = {"*.vide"};
				FileDialog fileDialog = new FileDialog(GenerateOclDialog.this.getShell(), SWT.OPEN);
				fileDialog.setFilterExtensions(filterExtensions);
				
				// sciezka do projektu
				IEditorInput editorInput = ((IFileEditorInput) PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor().getEditorInput()); 
				IProject project = ((IFileEditorInput) editorInput).getFile().getProject();
				fileDialog.setFilterPath(project.getLocation().toOSString());
				
				// sciezka do workspace
				//fileDialog.setFilterExtensions(filterExtensions);
				//fileDialog.setFilterPath(ResourcesPlugin.getWorkspace().getRoot().getLocation().toOSString());
				
				fileString = fileDialog.open();
				if (fileString != null) {
					fileText.setText(fileString);
				}
			}
		});
		
		fileText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				validateFinish();
			}
		});
		
		macroText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				validateFinish();
			}
		});
		
		setControl(topLevel);
		setPageComplete(false);
		validateFinish();
	}
	
	private void validateFinish() {
		if (!macroText.getText().equals("") && !fileText.getText().equals("")) {
			setPageComplete(true);
		} else {
			setPageComplete(false);
		}
	}
	
	public String getFileName() {
		return fileString;
	}
	
	public void setFileName(String fileName) {
		if (fileName != null)
			this.fileString = fileName;
		else
			this.fileString = "";
	}
	
	public String getMacroName() {
		return this.macroText.getText();
	}
	
	public void setMacroName(String macroName) {
		if (macroName != null)
			this.macroString = macroName;
		else
			this.macroString = "";
	}
}