package oqbe.exporters;

import java.util.ArrayList;

import org.eclipse.emf.common.util.EList;
import org.eclipse.ocl.types.BagType;
import org.eclipse.ocl.types.OrderedSetType;
import org.eclipse.ocl.types.SequenceType;
import org.eclipse.ocl.types.SetType;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Parameter;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Type;

import vide.ast.abstracts.AbstractNode;
import vide.ast.declarations.VariableDeclarationCS;
import vide.ast.editor.statements.VideEditorCode;
import vide.ast.expressions.ArgumentsCS;
import vide.ast.expressions.AttributeCallCS;
import vide.ast.expressions.AttributeOrVariableCS;
import vide.ast.expressions.BinaryExpressionCS;
import vide.ast.expressions.CollectionOperationCS;
import vide.ast.expressions.OclExpressionCS;
import vide.ast.expressions.OperationCallCS;
import vide.ast.expressions.Operator;
import vide.ast.expressions.UnaryExpressionCS;
import vide.ast.expressions.identifiers.CollectionMethodIdentifierCS;
import vide.ast.expressions.identifiers.CollectionTypeIdentifierCS;
import vide.ast.expressions.literals.CollectionLiteralExpCS;
import vide.ast.expressions.literals.IntegerLiteralExpCS;
import vide.ast.expressions.literals.TupleLiteralExpCS;
import vide.ast.expressions.literals.parts.CollectionLiteralPartCS;
import vide.ast.expressions.literals.parts.CollectionLiteralPartsCS;
import vide.ast.names.SimpleNameCS;
import vide.converters.VideCodeToAST;
import OQBE.Predicate;

public class ASTHelper {

	public static SimpleNameCS createSN(String text) {
		return new SimpleNameCS(text);
	}

	public static AttributeOrVariableCS createAttrOrVar(SimpleNameCS name, Element umlElement) {
		AttributeOrVariableCS result = new AttributeOrVariableCS(name);
		result.umlElement = umlElement;
		return result;
	}

	public static AttributeOrVariableCS createAttrOrVar(String name, Element umlElement) {
		return createAttrOrVar(new SimpleNameCS(name), umlElement);
	}

	public static AttributeCallCS createAttrCall(SimpleNameCS variable, String attribute, Element attrElement) {
		AttributeCallCS result = new AttributeCallCS(createAttrOrVar(variable, null), 
				createAttrOrVar(attribute, null));
		result.umlElement = attrElement;
		return result;
	}

	public static AttributeCallCS createAttrCall(String variable, String attribute, Element attrElement) {
		SimpleNameCS varName = createSN(variable);
		return createAttrCall(varName, attribute, attrElement);
	}

	public static AttributeCallCS createAttrCall(SimpleNameCS variable, String attribute1, String attribute2, Element attr2Element) {
		AttributeCallCS attrCall = new AttributeCallCS(createAttrOrVar(variable, null), 
				createAttrOrVar(attribute1, null));
		AttributeCallCS result = new AttributeCallCS(attrCall, 
				createAttrOrVar(attribute2, attr2Element));
		result.umlElement = attr2Element;
		return result;
	}

	public static AttributeCallCS createAttrCall(String variable, String attribute1, String attribute2, Element attr2Element) {
		SimpleNameCS varName = new SimpleNameCS(variable);
		return createAttrCall(varName, attribute1, attribute2, attr2Element);
	}

	public static AttributeCallCS createAttrCall(OclExpressionCS exp, String attribute) {
		AttributeCallCS result = new AttributeCallCS(exp, createAttrOrVar(attribute, null));
		return result;
	}

	public static CollectionLiteralExpCS createBagLiteral(OclExpressionCS expression) {
		CollectionLiteralPartCS part = new CollectionLiteralPartCS(expression);
		CollectionLiteralPartsCS parts = new CollectionLiteralPartsCS(part, null);
		CollectionTypeIdentifierCS collTypeId = CollectionTypeIdentifierCS.ctBag;
		return new CollectionLiteralExpCS(collTypeId, parts);
	}

	public static OclExpressionCS createVoAoAC(OclExpressionCS rootExp, EList<String> elements, Element umlElement) {
		OclExpressionCS result;
		if (rootExp == null)
			result = createAttrOrVar(elements.get(0), null);
		else
			result = createAttrCall(rootExp, elements.get(0));
		result.umlElement = umlElement;
		elements.remove(0);
		if (elements.size() > 0)
			return createVoAoAC(result, elements, umlElement);
		else
			return result;
	}

	public static OclExpressionCS createVoAoAC(EList<String> elements, Element umlElement) {
		return createVoAoAC(null, elements, umlElement);
	}

	public static OclExpressionCS createVoAoAC(String[] elements, Element umlElement) {
		NotNullEList<String> list = new NotNullEList<String>();
		for (String element : elements)
			list.add(element);
		return createVoAoAC(null, list, umlElement);
	}

	public static OclExpressionCS createVoAoAC(String[] elements) {
		return createVoAoAC(elements, null);
	}

	public static OclExpressionCS createVoAoAC(EList<String> elements) {
		// creates AttributeOrVariableCS or (multilevel) AttributeCallCS
		return createVoAoAC(elements, null);
	}
	
	private static Operator createOperator(String opSign) throws Exception {
		Operator result = Operator.getOperator(opSign);
		if (result == null)
			throw new Exception("Invalid operator! (" + opSign + ")");
		return result;
	}

	//
	
	public static OclExpressionCS createValueExp(Predicate predicate) throws Exception {
		String valueAsString = predicate.getArgument();
		if ((valueAsString == null) || valueAsString.equals(""))
		//	throw new Exception("Unassigned predicate argument!\n"
		//			+ "Predicate: " + predicate.getPredicateName() + ".");
			return null;
		else {
			VideEditorCode code = (VideEditorCode) VideCodeToAST.convert(valueAsString + ";");
			return (OclExpressionCS) code.statements.get(0);
		}
	}	
	
	public static OclExpressionCS createBinExp(Predicate predicate, String expPrefix) throws Exception {
		//AttributeOrVariableCS attr = ASTHelper.createAttrOrVar(predicate.getPredicateName(), null);
		OclExpressionCS attr = ASTHelper.createVoAoAC(new String[] {expPrefix, predicate.getPredicateName()});
		OclExpressionCS value = createValueExp(predicate);
		String opText = predicate.getOperator();
		if (value != null) {
			Operator operator = Operator.getOperator(opText);
			if (operator != null)
				return new BinaryExpressionCS(attr, value, operator);
			else {
				ArgumentsCS arguments = new ArgumentsCS(value, null);
				return new OperationCallCS(attr, ASTHelper.createSN(opText), arguments);
			}
		} else if (predicate.getOperator() != null) {			
			// collection operation?
			try {
				@SuppressWarnings("unused")
				int colMethodId = CollectionMethodIdentifierCS.getId(opText); // Is given operator valid?
				SimpleNameCS collMethod = ASTHelper.createSN(opText);
				return new CollectionOperationCS(attr, collMethod);
			} catch (NullPointerException e) {
				// unary expression?
				Operator operator = Operator.getOperator(opText);
				if (operator != null)
					return new UnaryExpressionCS(operator, attr);
				else
					// operation call?
					return new OperationCallCS(attr, ASTHelper.createSN(opText), null);
			}
		} else
			return attr;
	}

	public static OclExpressionCS createBinExpTree(EList<Predicate> predicates, int idx, String expPrefix) throws Exception {
		if (idx == predicates.size() - 1) {
			Predicate predicate = predicates.get(idx);
			OclExpressionCS result = createBinExp(predicate, expPrefix);
			return result;
		} else {
			BinaryExpressionCS result = new BinaryExpressionCS(null, null, Operator.opAnd);
			result.expression1 = createBinExp(predicates.get(idx), expPrefix);			
			result.expression2 = createBinExpTree(predicates, idx + 1, expPrefix);
			result.updateChildPointers();
			return result;
		}
	}	
	
	//
		
	protected static boolean isMultivalued(Type type) {
		return  (type instanceof BagType) ||
				(type instanceof OrderedSetType) || 
				(type instanceof SetType) ||
				(type instanceof SequenceType);
	}
	
	protected static boolean isMultivalued(Type type, org.eclipse.uml2.uml.Variable var) {
		return isMultivalued(type) || (var.getLower() != 1) || (var.getUpper() != 1);
	}

	@SuppressWarnings("unchecked")

	private static boolean isMultivalued(Element element) {
		boolean result = false; 
		if (element instanceof org.eclipse.uml2.uml.Class)
			result = false;
		else if (element instanceof Property) {
			Property prop = (Property) element;
			result = prop.isMultivalued();
			if ( (!result) && (prop.getLower() != prop.getUpper()) )
				result = true;
		} else if (element instanceof org.eclipse.ocl.expressions.Variable) {
			org.eclipse.ocl.expressions.Variable<Classifier, Parameter> var = (org.eclipse.ocl.expressions.Variable<Classifier, Parameter>) element;
			result = isMultivalued((Type) var.getType());
		} else if (element instanceof org.eclipse.uml2.uml.Variable) {
			org.eclipse.uml2.uml.Variable var = (org.eclipse.uml2.uml.Variable) element;
			result = isMultivalued(var.getType(), var);
		} else if (element instanceof Operation) {
			Parameter opResult = ((Operation) element).getReturnResult();
			if (opResult != null)
				result = isMultivalued(opResult.getType());
			else
				result = false;
		} 
		return result;
	}
	
	public static OclExpressionCS packExpression(OclExpressionCS expression) {
		if (expression == null)
			return expression;
		else {
			boolean createBag = false;
			if ( !(expression instanceof CollectionOperationCS) ) {
				if (expression.umlElement != null)
					createBag = !isMultivalued(expression.umlElement);
				else
					createBag = true;
			}
			if (createBag)
				return ASTHelper.createBagLiteral(expression);
			else
				return expression;
		}
	}

	public static AbstractNode createFakeASTNode() {
		return new IntegerLiteralExpCS(-1);
	}
	
	public static String lastVarName(TupleLiteralExpCS tuple) {
		String result = null;
		if (tuple != null) {
			ArrayList<VariableDeclarationCS> vars = tuple.varDeclList.declarations;
			result = vars.get(vars.size() - 1).name.value;
		}
		return result;
	}
	
}
