package oqbe.exporters;

import java.util.Stack;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;

import OQBE.Example;
import OQBE.LinkExample;

public class OQBEIterator {

	private OQBEModel model;
	private Stack<LinkExample> stack;
	private EList<Example> visited;
	private Example current;	
	private boolean prevVisited;
	private boolean prevNested;
	private EList<LinkExample> links;
	private EList<Example> allExamples;
	private boolean standaloneExample;
	
	private CallType callType;
	
	public OQBEIterator(OQBEModel model, Example example, CallType callType) {		
		stack = new Stack<LinkExample>();
		visited = new BasicEList<Example>();
		allExamples = model.getExamples();
		this.model = model;
		current = example;
		this.callType = callType;
	}
		
	private boolean isLastLink(LinkExample link) {
		EList<LinkExample> links = model.getLinkedExamples(current);
		return links.indexOf(link) == links.size() - 1;
	}
	
	private boolean checkPrevVisited() {
		boolean result = prevVisited;
		prevVisited = false;
		return result;
	}

	private Example goBack() {
		if (stack.size() == 0) {
			current = null;
			return null;
		} else {
			LinkExample lastLink = stack.peek();
			current = lastLink.getSource();
			if (isLastLink(lastLink)) {
				stack.pop();
				return goBack();
			} else
				return getNext();
		}
	}
	
	private Example goForward(int linkNo) {
		LinkExample link = links.get(linkNo);
		stack.push(link);
		if (model.linkToNestedOutput(link))
			prevNested = true;
		return link.getTarget();	
	}
	
	private Example firstNotVisited() {
		Example result = null;
		int idx = 0;
		while ( (result == null) && (idx < allExamples.size()) ) {
			Example example = allExamples.get(idx);
			if (visited.contains(example))
				idx ++;
			else
				result = example;
		}
		return result;
	}
	
	//
	
	public Example getNext() {
		Example result = null;
		// if we had standalone example previously => there are no more linked examples to visit 
		// (standalone, not visited, examples remain - maybe) 
		if ( (!standaloneExample /*previously*/) && (!prevNested)) {
			if (checkPrevVisited())
				result = goBack();
			else {
				boolean start = (stack.size() == 0) && (visited.size() == 0); 
				visited.add(current);
				links = model.getLinkedExamples(current);			
				if (start) {
					start = false;
					if (links.size() > 0)
						result = goForward(0);
				} else {
					LinkExample prevLink = stack.peek();
					int idx = links.indexOf(prevLink);
					if (idx == -1) // new example - get first link
						idx = 0; 
					else { // get next link, remove current from stack
						stack.pop();
						idx ++;
					}
					if (idx <= links.size() - 1) 
						result = goForward(idx);										
					else 
						result = goBack();								
				}
				current = result;
			}
			prevVisited = alreadyVisited();
		}
		// standalone examples?
		if ( (result == null) && (callType == CallType.STANDARD) ) {			
			result = firstNotVisited();
			if (result != null) {
				standaloneExample = true;
				visited.add(result);
			}
		}
		return result;
	}
	
	public boolean alreadyVisited() {
		return visited.indexOf(current) != -1;
	}
	
	public LinkExample getCurrentLink() {
		if (standaloneExample)
			return null;
		else
			return stack.peek();
	}
			
}
