package oqbe.wizards;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;

import model.repository.helpers.uml.DefineMethodBodyException;
import oqbe.dialogs.GenerateOclDialog;
import oqbe.exporters.ASTExporter;
import oqbe.exporters.OQBEModel;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IImportWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.part.FileEditorInput;

import vide.ast.abstracts.AbstractNode;
import vide.converters.ASTToVideCode;
import editor.plugin.textual.VideTextualEditor;


public class GenerateOclWizard extends Wizard implements IImportWizard {
	private GenerateOclDialog dialog;
	private IEditorPart editor;
	private IFile file;
	private String macroName;
	private String macroCode;
	private String diagramPath;
	private OQBEModel rep;
	
	public boolean performFinish() {
		final String filePathString = dialog.getFileName();
		macroName = dialog.getMacroName();
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		IFile[] files = root.findFilesForLocation(new Path(filePathString));
		if (files.length > 0)
			this.file = files[0];
		else {
			MessageDialog.openError(getShell(), "Error", "Invalid file path");
			return false;
		}
		
		IRunnableWithProgress op = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException {
				try {
					doFinish(monitor);
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				} catch (DefineMethodBodyException e) {
					throw new InvocationTargetException(e);					
				} finally {
					monitor.done();
				}
			}
		};
		
		try {
			getContainer().run(true, false, op);
		} catch (InterruptedException e) {
			return false;
		} catch (InvocationTargetException e) {
			Throwable realException = e.getTargetException();
			MessageDialog.openError(getShell(), "Error", realException.getMessage());
			return false;
		}		
		return true;
	}
		
	private void doFinish(IProgressMonitor monitor) throws CoreException, DefineMethodBodyException {
		monitor.beginTask("Generating OCL...", 4);
		FileEditorInput fileInput = (FileEditorInput) this.editor.getEditorInput();
		try {/*
			monitor.setTaskName("Loading model...");
			OQBEModel rep  = new OQBEModel(fileInput);
			monitor.worked(1);
			*/
			monitor.setTaskName("Creating AST...");	
			ASTExporter generator = new ASTExporter();
			AbstractNode node = generator.generate(rep);
			if (node != null) {
				macroCode = ASTToVideCode.convert(node, false);
				diagramPath = fileInput.getName();
				monitor.worked(1);
				
				monitor.setTaskName("Generating macro...");
				//macroCode = this.generateMacro(macroCode, macroName, diagramPath);
				monitor.worked(2);
				
				monitor.setTaskName("Creating file...");
				createFile(monitor);
				monitor.worked(3);
				
				monitor.setTaskName("Opening file...");
				openFile();
				monitor.worked(4);
			}
		} catch (final Exception e) {
			e.printStackTrace();
			getShell().getDisplay().asyncExec(new Runnable() {
				public void run() {
					MessageDialog.openError(editor.getSite().getShell(), "Error", e.getMessage());	
				}
			});
		}
	}
	/*
	private void writeFile(String macroCode, IProgressMonitor monitor) throws IOException {
		try {
			InputStream stream = new ByteArrayInputStream(macroCode.getBytes());
			
			if (!this.file.exists()) {
				String text = readFile();
				//MacroPreprocessor.setMacroCode(macroName, diagramPath, macroCode, text)
				
				//this.file.appendContents(stream, true, true, monitor);
			} else {
				this.file.create(stream, true, monitor);
			}
			
			stream.close();
			
		} catch (Exception e) {
		}
    }
	*/
	private void createFile(IProgressMonitor monitor) throws IOException, CoreException {
		InputStream stream = new ByteArrayInputStream("".getBytes());
		if (!this.file.exists()) {
			this.file.create(stream, true, monitor);
		}
		stream.close();
	}
	/*
	private String readFile() throws CoreException, IOException {
		BufferedReader reader = new BufferedReader(new InputStreamReader(this.file.getContents()));
		String text = "";
		String line = "";
		while ((line = reader.readLine()) != null) {
			text += line + "\n";
		}
		reader.close();
		
		return text;
	}
	*/
	protected void openFile() {
    	getShell().getDisplay().asyncExec(new Runnable() {
			public void run() {
				IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
				try {
					IEditorPart textualEditor = IDE.openEditor(page, GenerateOclWizard.this.file, true);
					if (textualEditor instanceof VideTextualEditor) {
						((VideTextualEditor) textualEditor).setMacro(macroName, diagramPath, macroCode);
					}
					
				} catch (PartInitException e) {
				}
			}
		});
    }
    
    //private String generateMacro(String code, String macroName, String diagramPath) {
    	//String header = "\n/*\n\tGenerated OQBE code\n*/\n";
    	//String macro = header + "<macro name=" +
    		//"\"" + macroName + "\"" +
    		//", veb=" +
    		//"\"" + "diagram:/" + diagramPath + "\"" +
    		//">\n" +
    		//"\t" + code + "\n" +
    		//"</macro>\n";
//    	
  //  	return macro;
    //}
    
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		setNeedsProgressMonitor(true);
		//this.selection = selection;
		this.editor = workbench.getActiveWorkbenchWindow().getActivePage().getActiveEditor();
	}
	
	public void addPages() {
		super.addPages();
		
		dialog = new GenerateOclDialog();
		addPage(dialog);
		
		FileEditorInput fileInput = (FileEditorInput) this.editor.getEditorInput();
		try {
			rep  = new OQBEModel(fileInput);
			this.dialog.setMacroName(rep.getPackage().getMacroName());
			if (rep.getPackage().getContext() != null)
				this.dialog.setFileName(ResourcesPlugin.getWorkspace().getRoot().getLocation() + rep.getPackage().getContext().getFilePath());
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}